@@ -26,10 +26,11 @@ import java.lang.reflect.Type;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.HashMap;
+import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 
-import javax.ws.rs.ConsumeMime;
-import javax.ws.rs.ProduceMime;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.ext.ContextResolver;
 import javax.ws.rs.ext.ExceptionMapper;
@@ -37,6 +38,7 @@ import javax.ws.rs.ext.MessageBodyReader;
 import javax.ws.rs.ext.MessageBodyWriter;
 
 import org.apache.cxf.jaxrs.ext.MappingsHandler;
+import org.apache.cxf.jaxrs.ext.ParameterHandler;
 import org.apache.cxf.jaxrs.ext.RequestHandler;
 import org.apache.cxf.jaxrs.ext.ResponseHandler;
 import org.apache.cxf.jaxrs.impl.RequestPreprocessor;
@@ -48,35 +50,47 @@ import org.apache.cxf.message.Message;
 
 public final class ProviderFactory {
     
-    private static final ProviderFactory PF = new ProviderFactory();
+    private static final Map<String, ProviderFactory> FACTORIES = 
+        new HashMap<String, ProviderFactory>();
+    private static final ProviderFactory DEFAULT_FACTORY = new ProviderFactory(); 
+    private static final String SLASH = "/"; 
     
     private List<ProviderInfo<MessageBodyReader>> defaultMessageReaders = 
         new ArrayList<ProviderInfo<MessageBodyReader>>();
     private List<ProviderInfo<MessageBodyWriter>> defaultMessageWriters = 
         new ArrayList<ProviderInfo<MessageBodyWriter>>();
     private List<ProviderInfo<MessageBodyReader>> userMessageReaders = 
-        new ArrayList<ProviderInfo<MessageBodyReader>>();
+        new ArrayList<ProviderInfo<MessageBodyReader>>(1);
     private List<ProviderInfo<MessageBodyWriter>> userMessageWriters = 
-        new ArrayList<ProviderInfo<MessageBodyWriter>>();
+        new ArrayList<ProviderInfo<MessageBodyWriter>>(1);
     private List<ProviderInfo<ContextResolver>> userContextResolvers = 
-        new ArrayList<ProviderInfo<ContextResolver>>();
+        new ArrayList<ProviderInfo<ContextResolver>>(1);
+    private List<ProviderInfo<ExceptionMapper>> defaultExceptionMappers = 
+        new ArrayList<ProviderInfo<ExceptionMapper>>(1);
     private List<ProviderInfo<ExceptionMapper>> userExceptionMappers = 
-        new ArrayList<ProviderInfo<ExceptionMapper>>();
+        new ArrayList<ProviderInfo<ExceptionMapper>>(1);
     private List<ProviderInfo<RequestHandler>> requestHandlers = 
-        new ArrayList<ProviderInfo<RequestHandler>>();
+        new ArrayList<ProviderInfo<RequestHandler>>(1);
     private List<ProviderInfo<ResponseHandler>> responseHandlers = 
-        new ArrayList<ProviderInfo<ResponseHandler>>();
+        new ArrayList<ProviderInfo<ResponseHandler>>(1);
+    private List<ProviderInfo<ParameterHandler>> jaxrsParamHandlers = 
+        new ArrayList<ProviderInfo<ParameterHandler>>(1);
     private RequestPreprocessor requestPreprocessor;
     
     private ProviderFactory() {
         // TODO : this needs to be done differently,
         // we need to use cxf-jaxrs-extensions
+        
+        // TODO : make sure the default providers are shared between multiple
+        // factories
+        
         setProviders(defaultMessageReaders,
                      defaultMessageWriters,
                      userContextResolvers,
                      requestHandlers,
                      responseHandlers,
-                     userExceptionMappers,
+                     defaultExceptionMappers,
+                     jaxrsParamHandlers,
                      new JAXBElementProvider(),
                      new JSONProvider(),
                      new BinaryDataProvider(),
@@ -84,12 +98,29 @@ public final class ProviderFactory {
                      new SourceProvider(),
                      new FormEncodingReaderProvider(),
                      new PrimitiveTextProvider(),
+                     new ActivationProvider(),
                      new WebApplicationExceptionMapper(),
                      new MappingsHandler());
     }
     
     public static ProviderFactory getInstance() {
-        return PF;
+        return getInstance("/");
+    }
+    
+    public static ProviderFactory getInstance(String baseAddress) {
+        if (SLASH.equals(baseAddress)) {
+            return DEFAULT_FACTORY;
+        }
+        
+        ProviderFactory pf = null;
+        synchronized (ProviderFactory.class) { 
+            pf = FACTORIES.get(baseAddress);
+            if (pf == null) {
+                pf = new ProviderFactory();
+                FACTORIES.put(baseAddress, pf);
+            }
+        }
+        return pf;
     }
 
     @SuppressWarnings("unchecked")
@@ -114,9 +145,27 @@ public final class ProviderFactory {
         return null;
     }
     
-    @SuppressWarnings("unchecked")
     public <T> ExceptionMapper<T> createExceptionMapper(Class<?> exceptionType, Message m) {
-        for (ProviderInfo<ExceptionMapper> em : userExceptionMappers) {
+        
+        ExceptionMapper<T> mapper = doCreateExceptionMapper(userExceptionMappers,
+                                                            exceptionType,
+                                                            m);
+        if (mapper != null) {
+            return mapper;
+        }
+        
+        return doCreateExceptionMapper(defaultExceptionMappers,
+                                       exceptionType,
+                                       m);
+    }
+    
+    @SuppressWarnings("unchecked")
+    private static <T> ExceptionMapper<T> doCreateExceptionMapper(
+        List<ProviderInfo<ExceptionMapper>> mappers, Class<?> exceptionType, Message m) {
+        
+        List<ExceptionMapper<T>> candidates = new LinkedList<ExceptionMapper<T>>();
+        
+        for (ProviderInfo<ExceptionMapper> em : mappers) {
             Type[] types = em.getProvider().getClass().getGenericInterfaces();
             for (Type t : types) {
                 if (t instanceof ParameterizedType) {
@@ -126,13 +175,43 @@ public final class ProviderFactory {
                         if (((Class<?>)args[i]).isAssignableFrom(exceptionType)) {
                             InjectionUtils.injectContextFields(em.getProvider(), em, m);
                             InjectionUtils.injectContextMethods(em.getProvider(), em, m);
-                            return em.getProvider();
+                            candidates.add(em.getProvider());
                         }
                     }
                 }
             }
         }
-        return null;
+        if (candidates.size() == 0) {
+            return null;
+        }
+        Collections.sort((List)candidates, new ExceptionMapperComparator());
+        return candidates.get(0);
+    }
+    
+    @SuppressWarnings("unchecked")
+    public <T> ParameterHandler<T> createParameterHandler(Class<?> paramType) {
+        
+        List<ParameterHandler<T>> candidates = new LinkedList<ParameterHandler<T>>();
+        
+        for (ProviderInfo<ParameterHandler> em : jaxrsParamHandlers) {
+            Type[] types = em.getProvider().getClass().getGenericInterfaces();
+            for (Type t : types) {
+                if (t instanceof ParameterizedType) {
+                    ParameterizedType pt = (ParameterizedType)t;
+                    Type[] args = pt.getActualTypeArguments();
+                    for (int i = 0; i < args.length; i++) {
+                        if (((Class<?>)args[i]).isAssignableFrom(paramType)) {
+                            candidates.add(em.getProvider());
+                        }
+                    }
+                }
+            }
+        }
+        if (candidates.size() == 0) {
+            return null;
+        }
+        Collections.sort(candidates, new ParameterHandlerComparator());
+        return candidates.get(0);
     }
     
     public <T> MessageBodyReader<T> createMessageBodyReader(Class<T> bodyType,
@@ -199,13 +278,14 @@ public final class ProviderFactory {
         return mw;
     }
     
-       
+//CHECKSTYLE:OFF       
     private void setProviders(List<ProviderInfo<MessageBodyReader>> readers, 
                               List<ProviderInfo<MessageBodyWriter>> writers,
                               List<ProviderInfo<ContextResolver>> resolvers,
                               List<ProviderInfo<RequestHandler>> requestFilters,
                               List<ProviderInfo<ResponseHandler>> responseFilters,
                               List<ProviderInfo<ExceptionMapper>> excMappers,
+                              List<ProviderInfo<ParameterHandler>> paramHandlers,
                               Object... providers) {
         
         for (Object o : providers) {
@@ -232,6 +312,10 @@ public final class ProviderFactory {
             if (ExceptionMapper.class.isAssignableFrom(o.getClass())) {
                 excMappers.add(new ProviderInfo<ExceptionMapper>((ExceptionMapper)o)); 
             }
+            
+            if (ParameterHandler.class.isAssignableFrom(o.getClass())) {
+                paramHandlers.add(new ProviderInfo<ParameterHandler>((ParameterHandler)o)); 
+            }
         }
         
         sortReaders(readers);
@@ -239,6 +323,7 @@ public final class ProviderFactory {
         
         injectContexts(readers, writers, resolvers, requestFilters, responseFilters, excMappers);
     }
+//CHECKSTYLE:ON
     
     void injectContexts(List<?> ... providerLists) {
         for (List<?> list : providerLists) {
@@ -285,10 +370,9 @@ public final class ProviderFactory {
                                                          MediaType mediaType,
                                                          Message m) {
         for (ProviderInfo<MessageBodyReader> ep : readers) {
-            
+            InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
+            InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);             
             if (matchesReaderCriterias(ep.getProvider(), type, genericType, annotations, mediaType)) {
-                InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
-                InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);
                 return ep.getProvider();
             }
         }     
@@ -306,12 +390,10 @@ public final class ProviderFactory {
             return false;
         }
         
-        List<MediaType> supportedMediaTypes =
-            JAXRSUtils.getConsumeTypes(ep.getClass().getAnnotation(ConsumeMime.class));
+        List<MediaType> supportedMediaTypes = JAXRSUtils.getProviderConsumeTypes(ep);
         
         List<MediaType> availableMimeTypes = 
-            JAXRSUtils.intersectMimeTypes(Collections.singletonList(mediaType),
-                                          supportedMediaTypes);
+            JAXRSUtils.intersectMimeTypes(Collections.singletonList(mediaType), supportedMediaTypes);
 
         return availableMimeTypes.size() != 0 ? true : false;
         
@@ -336,9 +418,9 @@ public final class ProviderFactory {
                                                          MediaType mediaType,
                                                          Message m) {
         for (ProviderInfo<MessageBodyWriter> ep : writers) {
+            InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
+            InjectionUtils.injectContextMethods(ep.getProvider(), ep, m); 
             if (matchesWriterCriterias(ep.getProvider(), type, genericType, annotations, mediaType)) {
-                InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
-                InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);
                 return ep.getProvider();
             }
         }     
@@ -356,8 +438,7 @@ public final class ProviderFactory {
             return false;
         }
         
-        List<MediaType> supportedMediaTypes =
-            JAXRSUtils.getProduceTypes(ep.getClass().getAnnotation(ProduceMime.class));
+        List<MediaType> supportedMediaTypes = JAXRSUtils.getProviderProduceTypes(ep);
         
         List<MediaType> availableMimeTypes = 
             JAXRSUtils.intersectMimeTypes(Collections.singletonList(mediaType),
@@ -367,25 +448,6 @@ public final class ProviderFactory {
         
     }
     
-    public boolean deregisterEntityProvide(Object o) {
-        
-        if (o instanceof MessageBodyReader) {
-            return userMessageReaders.remove(o);
-        }
-        if (o instanceof MessageBodyWriter) {
-            return userMessageWriters.remove(o);
-        }
-        if (o instanceof ContextResolver) {
-            return userContextResolvers.remove(o);
-        }
-        if (o instanceof RequestHandler) {
-            return requestHandlers.remove(o);
-        }
-        
-        return false;
-                                               
-    }
-    
     List<ProviderInfo<MessageBodyReader>> getDefaultMessageReaders() {
         return Collections.unmodifiableList(defaultMessageReaders);
     }
@@ -421,6 +483,7 @@ public final class ProviderFactory {
                      requestHandlers,
                      responseHandlers,
                      userExceptionMappers,
+                     jaxrsParamHandlers,
                      userProviders.toArray());
     }
 
@@ -431,32 +494,14 @@ public final class ProviderFactory {
                            ProviderInfo<MessageBodyReader> p2) {
             MessageBodyReader e1 = p1.getProvider();
             MessageBodyReader e2 = p2.getProvider();
-            
-            ConsumeMime c = e1.getClass().getAnnotation(ConsumeMime.class);
-            String[] mimeType1 = {"*/*"};
-            if (c != null) {
-                mimeType1 = c.value();               
-            }
-            
-            ConsumeMime c2 = e2.getClass().getAnnotation(ConsumeMime.class);
-            String[] mimeType2 = {"*/*"};
-            if (c2 != null) {
-                mimeType2 = c2.value();               
-            }
+            List<MediaType> types1 = JAXRSUtils.getProviderConsumeTypes(e1);
+            types1 = JAXRSUtils.sortMediaTypes(types1);
+            List<MediaType> types2 = JAXRSUtils.getProviderConsumeTypes(e2);
+            types2 = JAXRSUtils.sortMediaTypes(types2);
     
-            return compareString(mimeType1[0], mimeType2[0]);
+            return JAXRSUtils.compareSortedMediaTypes(types1, types2);
             
         }
-
-        private int compareString(String str1, String str2) {
-            if (!str1.startsWith("*/") && str2.startsWith("*/")) {
-                return -1;
-            } else if (str1.startsWith("*/") && !str2.startsWith("*/")) {
-                return 1;
-            } 
-            
-            return str1.compareTo(str2);
-        }
     }
     
     private static class MessageBodyWriterComparator 
@@ -467,34 +512,17 @@ public final class ProviderFactory {
             MessageBodyWriter e1 = p1.getProvider();
             MessageBodyWriter e2 = p2.getProvider();
             
-            ProduceMime c = e1.getClass().getAnnotation(ProduceMime.class);
-            String[] mimeType1 = {"*/*"};
-            if (c != null) {
-                mimeType1 = c.value();               
-            }
-            
-            ProduceMime c2 = e2.getClass().getAnnotation(ProduceMime.class);
-            String[] mimeType2 = {"*/*"};
-            if (c2 != null) {
-                mimeType2 = c2.value();               
-            }
+            List<MediaType> types1 =
+                JAXRSUtils.sortMediaTypes(JAXRSUtils.getProviderProduceTypes(e1));
+            List<MediaType> types2 =
+                JAXRSUtils.sortMediaTypes(JAXRSUtils.getProviderProduceTypes(e2));
     
-            return compareString(mimeType1[0], mimeType2[0]);
+            return JAXRSUtils.compareSortedMediaTypes(types1, types2);
             
         }
-        
-        private int compareString(String str1, String str2) {
-            if (!str1.startsWith("*/") && str2.startsWith("*/")) {
-                return -1;
-            } else if (str1.startsWith("*/") && !str2.startsWith("*/")) {
-                return 1;
-            } 
-            
-            return str1.compareTo(str2);
-        }
     }
     
-    public void setRequestPreporcessor(RequestPreprocessor rp) {
+    public void setRequestPreprocessor(RequestPreprocessor rp) {
         this.requestPreprocessor = rp;
     }
     
@@ -502,7 +530,7 @@ public final class ProviderFactory {
         return requestPreprocessor;
     }
     
-    public void cleatThreadLocalProxies() {
+    public void clearThreadLocalProxies() {
         clearProxies(defaultMessageReaders,
                      defaultMessageWriters,
                      userMessageReaders,
@@ -529,6 +557,7 @@ public final class ProviderFactory {
         userExceptionMappers.clear();
         requestHandlers.clear();
         responseHandlers.clear();
+        jaxrsParamHandlers.clear();
     }
     
     public void setSchemaLocations(List<String> schemas) {
@@ -548,4 +577,40 @@ public final class ProviderFactory {
             }
         }
     }
+    
+    private static class ExceptionMapperComparator implements 
+        Comparator<ExceptionMapper<? extends Throwable>> {
+
+        public int compare(ExceptionMapper<? extends Throwable> em1, 
+                           ExceptionMapper<? extends Throwable> em2) {
+            return compareClasses(em1.getClass(), em2.getClass());
+        }
+        
+    }
+    
+    private static class ParameterHandlerComparator implements 
+        Comparator<ParameterHandler<? extends Object>> {
+
+        public int compare(ParameterHandler<? extends Object> em1, 
+                           ParameterHandler<? extends Object> em2) {
+            return compareClasses(em1.getClass(), em2.getClass());
+        }
+    
+    }
+    
+    private static int compareClasses(Class<?> cl1, Class<?> cl2) {
+        Type[] types1 = cl1.getGenericInterfaces();
+        Type[] types2 = cl2.getGenericInterfaces();
+        
+        Class<?> realClass1 = InjectionUtils.getActualType(types1[0]);
+        Class<?> realClass2 = InjectionUtils.getActualType(types2[0]);
+        if (realClass1 == realClass2) {
+            return 0;
+        }
+        if (realClass1.isAssignableFrom(realClass2)) {
+            // subclass should go first
+            return 1;
+        }
+        return -1;
+    }
 }
