@@ -24,10 +24,13 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.UUID;
 
+import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
@@ -40,14 +43,16 @@ import javax.ws.rs.ext.MessageBodyWorkers;
 import javax.xml.bind.JAXBContext;
 
 import org.apache.cxf.jaxrs.Customer;
+import org.apache.cxf.jaxrs.CustomerGender;
+import org.apache.cxf.jaxrs.CustomerParameterHandler;
 import org.apache.cxf.jaxrs.JAXBContextProvider;
 import org.apache.cxf.jaxrs.JAXRSServiceFactoryBean;
 import org.apache.cxf.jaxrs.JAXRSServiceImpl;
 import org.apache.cxf.jaxrs.SimpleFactory;
 import org.apache.cxf.jaxrs.impl.HttpHeadersImpl;
-import org.apache.cxf.jaxrs.impl.MessageBodyWorkersImpl;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.impl.PathSegmentImpl;
+import org.apache.cxf.jaxrs.impl.ProvidersImpl;
 import org.apache.cxf.jaxrs.impl.RequestImpl;
 import org.apache.cxf.jaxrs.impl.SecurityContextImpl;
 import org.apache.cxf.jaxrs.impl.UriInfoImpl;
@@ -64,6 +69,7 @@ import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.transport.http.AbstractHTTPDestination;
 import org.easymock.EasyMock;
+
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
@@ -118,6 +124,23 @@ public class JAXRSUtilsTest extends Assert {
                      org.apache.cxf.jaxrs.resources.TestResourceTemplate2.class);
     }
     
+    @Test
+    public void testSelectBetweenMultipleResourceClasses3() throws Exception {
+        JAXRSServiceFactoryBean sf = new JAXRSServiceFactoryBean();
+        sf.setResourceClasses(org.apache.cxf.jaxrs.resources.TestResourceTemplate4.class,
+                              org.apache.cxf.jaxrs.resources.TestResourceTemplate3.class);
+        sf.create();        
+        List<ClassResourceInfo> resources = ((JAXRSServiceImpl)sf.getService()).getClassResourceInfos();
+        MultivaluedMap<String, String> map = new MetadataMap<String, String>();
+        ClassResourceInfo bStore = JAXRSUtils.selectResourceClass(resources, "/", map);
+        assertEquals(bStore.getResourceClass(), org.apache.cxf.jaxrs.resources.TestResourceTemplate3.class);
+        
+        bStore = JAXRSUtils.selectResourceClass(resources, "/test", map);
+        assertEquals(bStore.getResourceClass(), 
+                     org.apache.cxf.jaxrs.resources.TestResourceTemplate4.class);
+        
+    }
+    
     @Test
     public void testFindTargetResourceClass() throws Exception {
         JAXRSServiceFactoryBean sf = new JAXRSServiceFactoryBean();
@@ -420,6 +443,40 @@ public class JAXRSUtilsTest extends Assert {
                    JAXRSUtils.compareMediaTypes(m4, m3) < 0);
     }
     
+    @Test
+    public void testCompareSortedMediaTypes() throws Exception {
+        MediaType m1 = MediaType.valueOf("text/xml");
+        MediaType m2 = MediaType.valueOf("text/*");
+        assertTrue("text/xml is more specific than text/*", 
+                   JAXRSUtils.compareSortedMediaTypes(Collections.singletonList(m1), 
+                                                      Collections.singletonList(m2)) < 0);
+        assertTrue("text/* is less specific than text/xml", 
+                   JAXRSUtils.compareSortedMediaTypes(Collections.singletonList(m2), 
+                                                      Collections.singletonList(m1)) > 0);
+        
+        assertTrue("text/xml is the same as text/xml", 
+                   JAXRSUtils.compareSortedMediaTypes(Collections.singletonList(m1), 
+                                                      Collections.singletonList(m1)) == 0);
+        
+        List<MediaType> sortedList1 = new ArrayList<MediaType>();
+        sortedList1.add(m1);
+        sortedList1.add(m2);
+                
+        List<MediaType> sortedList2 = new ArrayList<MediaType>();
+        sortedList2.add(m1);
+        sortedList2.add(m2);
+        
+        assertTrue("lists should be equal", 
+                   JAXRSUtils.compareSortedMediaTypes(sortedList1, sortedList2) == 0);
+        
+        sortedList1.add(MediaType.valueOf("*/*"));
+        assertTrue("first list should be less specific", 
+                   JAXRSUtils.compareSortedMediaTypes(sortedList1, sortedList2) > 0);
+        sortedList1.add(MediaType.valueOf("*/*"));
+        assertTrue("second list should be more specific", 
+                   JAXRSUtils.compareSortedMediaTypes(sortedList2, sortedList1) < 0);
+    }
+    
     @Test
     public void testAcceptTypesMatch() throws Exception {
         
@@ -461,6 +518,77 @@ public class JAXRSUtilsTest extends Assert {
         
     }
     
+    @Test
+    public void testFromStringParameters() throws Exception {
+        Class[] argType = {UUID.class, CustomerGender.class, CustomerGender.class};
+        Method m = Customer.class.getMethod("testFromStringParam", argType);
+        UUID u = UUID.randomUUID();
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.QUERY_STRING, "p1=" + u.toString() + "&p2=1&p3=2");
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+        assertEquals(3, params.size());
+        assertEquals("Query UUID Parameter was not matched correctly", 
+                     u.toString(), params.get(0).toString());
+        assertSame(CustomerGender.FEMALE, params.get(1));
+        assertSame(CustomerGender.MALE, params.get(2));
+    }
+    
+    @Test
+    public void testCustomerParameter() throws Exception {
+        ProviderFactory.getInstance().registerUserProvider(
+            new CustomerParameterHandler());
+        Class[] argType = {Customer.class};
+        Method m = Customer.class.getMethod("testCustomerParam", argType);
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.QUERY_STRING, "p1=Fred");
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+        assertEquals(1, params.size());
+        Customer c = (Customer)params.get(0);
+        assertEquals("Fred", c.getName());
+    }
+    
+    @Test
+    public void testWrongType() throws Exception {
+        Class[] argType = {HashMap.class};
+        Method m = Customer.class.getMethod("testWrongType", argType);
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.QUERY_STRING, "p1=1");
+        try {
+            JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+            fail("HashMap can not be handled as parameter");
+        } catch (WebApplicationException ex) {
+            assertEquals(500, ex.getResponse().getStatus());
+            assertEquals("Parameter Class java.util.HashMap has no constructor with "
+                         + "single String parameter, static valueOf(String) or fromString(String) methods", 
+                         ex.getResponse().getEntity().toString());
+        }
+        
+    }
+    
+    @Test
+    public void testExceptionDuringConstruction() throws Exception {
+        Class[] argType = {CustomerGender.class};
+        Method m = Customer.class.getMethod("testWrongType2", argType);
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.QUERY_STRING, "p1=3");
+        try {
+            JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           null, 
+                                                           messageImpl);
+            fail("CustomerGender have no instance with name 3");
+        } catch (WebApplicationException ex) {
+            assertEquals(404, ex.getResponse().getStatus());
+        }
+        
+    }
+    
+    
     @Test
     public void testQueryParametersBean() throws Exception {
         Class[] argType = {Customer.CustomerBean.class};
@@ -499,6 +627,24 @@ public class JAXRSUtilsTest extends Assert {
         assertEquals(new Long(123), cb.getB());
     }
     
+    @Test
+    public void testMatrixParametersBean() throws Exception {
+        Class[] argType = {Customer.CustomerBean.class};
+        Method m = Customer.class.getMethod("testMatrixBean", argType);
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.REQUEST_URI, "/bar;a=aValue/baz;b=123");
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null),
+                                                           new MetadataMap<String, String>(), 
+                                                           messageImpl);
+        assertEquals("Bean should be created", 1, params.size());
+        Customer.CustomerBean cb = (Customer.CustomerBean)params.get(0);
+        assertNotNull(cb);
+        
+        assertEquals("aValue", cb.getA());
+        assertEquals(new Long(123), cb.getB());
+    }
+    
+    
     @Test
     public void testMultipleQueryParameters() throws Exception {
         Class[] argType = {String.class, String.class, Long.class, 
@@ -522,23 +668,54 @@ public class JAXRSUtilsTest extends Assert {
                      "", params.get(4));
     }
     
+    @SuppressWarnings("unchecked")
     @Test
     public void testMatrixParameters() throws Exception {
-        Class[] argType = {String.class, String.class};
+        Class[] argType = {String.class, String.class, String.class, String.class, List.class};
         Method m = Customer.class.getMethod("testMatrixParam", argType);
         MessageImpl messageImpl = new MessageImpl();
         
-        messageImpl.put(Message.PATH_INFO, "/foo/bar;p1=1;p2");
+        messageImpl.put(Message.REQUEST_URI, "/foo;p4=0;p3=3/bar;p1=1;p2/baz;p4=4;p4=5");
         List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
                                                            null, messageImpl);
-        assertEquals("2 Matrix params should've been identified", 2, params.size());
+        assertEquals("5 Matrix params should've been identified", 5, params.size());
         
-        assertEquals("First Matrix Parameter of multiple was not matched correctly", 
+        assertEquals("First Matrix Parameter not matched correctly", 
                      "1", params.get(0));
-        assertEquals("Second Matrix Parameter of multiple was not matched correctly", 
+        assertEquals("Second Matrix Parameter was not matched correctly", 
                      "", params.get(1));
+        assertEquals("Third Matrix Parameter was not matched correctly", 
+                     "3", params.get(2));
+        assertEquals("Fourth Matrix Parameter was not matched correctly", 
+                     "0", params.get(3));
+        List<String> list = (List<String>)params.get(4);
+        assertEquals(3, list.size());
+        assertEquals("0", list.get(0));
+        assertEquals("4", list.get(1));
+        assertEquals("5", list.get(2));
     }
     
+    @Test
+    public void testMatrixAndPathSegmentParameters() throws Exception {
+        Class[] argType = {PathSegment.class, String.class};
+        Method m = Customer.class.getMethod("testPathSegment", argType);
+        MessageImpl messageImpl = new MessageImpl();
+        messageImpl.put(Message.REQUEST_URI, "/bar%20foo;p4=0%201");
+        MultivaluedMap<String, String> values = new MetadataMap<String, String>();
+        values.add("ps", "bar%20foo;p4=0%201");
+        List<Object> params = JAXRSUtils.processParameters(new OperationResourceInfo(m, null), 
+                                                           values, 
+                                                           messageImpl);
+        assertEquals("2 params should've been identified", 2, params.size());
+        
+        PathSegment ps = (PathSegment)params.get(0);
+        assertEquals("bar foo", ps.getPath());
+        assertEquals(1, ps.getMatrixParameters().size());
+        assertEquals("0 1", ps.getMatrixParameters().getFirst("p4"));
+        assertEquals("bar foo", params.get(1));
+    }
+    
+    
     @Test
     public void testSelectResourceMethod() throws Exception {
         ClassResourceInfo cri = new ClassResourceInfo(Customer.class);
@@ -577,6 +754,7 @@ public class JAXRSUtilsTest extends Assert {
         assertSame(ori, ori2);
     }
     
+    @SuppressWarnings("unchecked")
     @Test
     public void testHttpContextParameters() throws Exception {
         
@@ -589,26 +767,51 @@ public class JAXRSUtilsTest extends Assert {
                                                      Request.class,
                                                      SecurityContext.class,
                                                      MessageBodyWorkers.class,
-                                                     String.class}), 
+                                                     String.class,
+                                                     List.class}), 
                 cri);
         ori.setHttpMethod("GET");
         MultivaluedMap<String, String> headers = new MetadataMap<String, String>();
-        headers.add("Foo", "bar");
-        headers.add("Foo", "baz");
+        headers.add("Foo", "bar, baz");
         
         Message m = new MessageImpl();
         m.put(Message.PROTOCOL_HEADERS, headers);
         
         List<Object> params = 
             JAXRSUtils.processParameters(ori, new MetadataMap<String, String>(), m);
-        assertEquals("6 parameters expected", 6, params.size());
+        assertEquals("7 parameters expected", 7, params.size());
         assertSame(UriInfoImpl.class, params.get(0).getClass());
         assertSame(HttpHeadersImpl.class, params.get(1).getClass());
         assertSame(RequestImpl.class, params.get(2).getClass());
         assertSame(SecurityContextImpl.class, params.get(3).getClass());
-        assertSame(MessageBodyWorkersImpl.class, params.get(4).getClass());
+        assertSame(ProvidersImpl.class, params.get(4).getClass());
         assertSame(String.class, params.get(5).getClass());
-        assertEquals("Wrong header param", "bar,baz", params.get(5));
+        assertEquals("Wrong header param", "bar", params.get(5));
+        List<String> values = (List<String>)params.get(6);
+        assertEquals("Wrong headers size", 2, values.size());
+        assertEquals("Wrong 1st header param", "bar", values.get(0));
+        assertEquals("Wrong 2nd header param", "baz", values.get(1));
+    }
+    
+    @Test
+    public void testHttpContextParametersFromInterface() throws Exception {
+        
+        ClassResourceInfo cri = new ClassResourceInfo(Customer.class, true);
+        Method methodToInvoke = 
+            Customer.class.getMethod("setUriInfoContext", 
+                                     new Class[]{UriInfo.class});
+        OperationResourceInfo ori = 
+            new OperationResourceInfo(methodToInvoke, cri);
+        ori.setHttpMethod("GET");
+        ori.setAnnotatedMethod(AnnotationUtils.getAnnotatedMethod(methodToInvoke));
+        
+        
+        Message m = new MessageImpl();
+        
+        List<Object> params = 
+            JAXRSUtils.processParameters(ori, new MetadataMap<String, String>(), m);
+        assertEquals("1 parameters expected", 1, params.size());
+        assertSame(UriInfoImpl.class, params.get(0).getClass());
     }
     
     @Test
@@ -620,27 +823,33 @@ public class JAXRSUtilsTest extends Assert {
                 Customer.class.getMethod("testServletParams", 
                                          new Class[]{HttpServletRequest.class, 
                                                      HttpServletResponse.class, 
-                                                     ServletContext.class}), 
+                                                     ServletContext.class,
+                                                     ServletConfig.class}), 
                 cri);
         ori.setHttpMethod("GET");
         HttpServletRequest request = EasyMock.createMock(HttpServletRequest.class);
         HttpServletResponse response = EasyMock.createMock(HttpServletResponse.class);
         ServletContext context = EasyMock.createMock(ServletContext.class);
+        ServletConfig config = EasyMock.createMock(ServletConfig.class);        
+        
         EasyMock.replay(request);
         EasyMock.replay(response);
         EasyMock.replay(context);
+        EasyMock.replay(config);
         
         Message m = new MessageImpl();
         m.put(AbstractHTTPDestination.HTTP_REQUEST, request);
         m.put(AbstractHTTPDestination.HTTP_RESPONSE, response);
         m.put(AbstractHTTPDestination.HTTP_CONTEXT, context);
+        m.put(AbstractHTTPDestination.HTTP_CONFIG, config);
         
         List<Object> params = 
             JAXRSUtils.processParameters(ori, new MetadataMap<String, String>(), m);
-        assertEquals("3 parameters expected", 3, params.size());
+        assertEquals("4 parameters expected", 4, params.size());
         assertSame(request.getClass(), params.get(0).getClass());
         assertSame(response.getClass(), params.get(1).getClass());
         assertSame(context.getClass(), params.get(2).getClass());
+        assertSame(config.getClass(), params.get(3).getClass());
         
     }
     
@@ -661,7 +870,7 @@ public class JAXRSUtilsTest extends Assert {
         assertSame(HttpHeadersImpl.class, c.getHeaders().getClass());
         assertSame(RequestImpl.class, c.getRequest().getClass());
         assertSame(SecurityContextImpl.class, c.getSecurityContext().getClass());
-        assertSame(MessageBodyWorkersImpl.class, c.getBodyWorkers().getClass());
+        assertSame(ProvidersImpl.class, c.getBodyWorkers().getClass());
         
     }
     
@@ -671,12 +880,19 @@ public class JAXRSUtilsTest extends Assert {
         ClassResourceInfo cri = new ClassResourceInfo(Customer.class, true);
         Customer c = new Customer();
         cri.setResourceProvider(new SingletonResourceProvider(c));
-        InjectionUtils.injectContextProxies(cri, cri.getResourceProvider().getInstance());
         
         Message m = new MessageImpl();
         m.put(Message.PROTOCOL_HEADERS, new HashMap<String, List<String>>());
+        ServletContext servletContextMock = EasyMock.createNiceMock(ServletContext.class);
+        m.put(AbstractHTTPDestination.HTTP_CONTEXT, servletContextMock);
+        HttpServletRequest httpRequest = EasyMock.createNiceMock(HttpServletRequest.class);
+        m.put(AbstractHTTPDestination.HTTP_REQUEST, httpRequest);
+        HttpServletResponse httpResponse = EasyMock.createNiceMock(HttpServletResponse.class);
+        m.put(AbstractHTTPDestination.HTTP_RESPONSE, httpResponse);
         
+        InjectionUtils.injectContextProxies(cri, cri.getResourceProvider().getInstance());
         InjectionUtils.injectContextFields(c, cri, m);
+        InjectionUtils.injectContextMethods(c, cri, m);
         assertSame(ThreadLocalUriInfo.class, c.getUriInfo2().getClass());
         assertSame(UriInfoImpl.class, 
                    ((ThreadLocalProxy)c.getUriInfo2()).get().getClass());
@@ -686,9 +902,43 @@ public class JAXRSUtilsTest extends Assert {
                    ((ThreadLocalProxy)c.getRequest()).get().getClass());
         assertSame(SecurityContextImpl.class, 
                    ((ThreadLocalProxy)c.getSecurityContext()).get().getClass());
-        assertSame(MessageBodyWorkersImpl.class, 
+        assertSame(ProvidersImpl.class, 
                    ((ThreadLocalProxy)c.getBodyWorkers()).get().getClass());
+
+        assertSame(servletContextMock, 
+                   ((ThreadLocalProxy)c.getThreadLocalServletContext()).get());
+
+        assertSame(servletContextMock, 
+                   ((ThreadLocalProxy)c.getServletContext()).get());
         
+        assertSame(httpRequest, 
+                   ((ThreadLocalProxy)c.getServletRequest()).get());
+        assertSame(httpResponse, 
+                   ((ThreadLocalProxy)c.getServletResponse()).get());
+    }
+    
+    @Test
+    public void testSingletonHttpResourceFields() throws Exception {
+        
+        ClassResourceInfo cri = new ClassResourceInfo(Customer.class, true);
+        Customer c = new Customer();
+        cri.setResourceProvider(new SingletonResourceProvider(c));
+                
+        Message m = new MessageImpl();
+        ServletContext servletContextMock = EasyMock.createNiceMock(ServletContext.class);
+        m.put(AbstractHTTPDestination.HTTP_CONTEXT, servletContextMock);
+        HttpServletRequest httpRequest = EasyMock.createNiceMock(HttpServletRequest.class);
+        m.put(AbstractHTTPDestination.HTTP_REQUEST, httpRequest);
+        HttpServletResponse httpResponse = EasyMock.createNiceMock(HttpServletResponse.class);
+        m.put(AbstractHTTPDestination.HTTP_RESPONSE, httpResponse);
+        InjectionUtils.injectContextProxies(cri, cri.getResourceProvider().getInstance());
+        InjectionUtils.injectResourceFields(c, cri, m);
+        assertSame(servletContextMock, 
+                   ((ThreadLocalProxy)c.getServletContextResource()).get());
+        assertSame(httpRequest, 
+                   ((ThreadLocalProxy)c.getServletRequestResource()).get());
+        assertSame(httpResponse, 
+                   ((ThreadLocalProxy)c.getServletResponseResource()).get());
     }
     
     @Test
@@ -753,7 +1003,7 @@ public class JAXRSUtilsTest extends Assert {
         ProviderFactory.getInstance().registerUserProvider(cr);
         
         Message m = new MessageImpl();
-            
+        m.put(Message.BASE_PATH, "/");    
         List<Object> params = 
             JAXRSUtils.processParameters(ori, new MetadataMap<String, String>(), m);
         assertEquals("1 parameters expected", 1, params.size());
@@ -772,7 +1022,7 @@ public class JAXRSUtilsTest extends Assert {
         ProviderFactory.getInstance().registerUserProvider(cr);
         
         Message m = new MessageImpl();
-            
+        m.put(Message.BASE_PATH, "/");    
         InjectionUtils.injectContextFields(c, ori.getClassResourceInfo(), m);
         assertSame(cr.getClass(), c.getContextResolver().getClass());
     }
