@@ -154,7 +154,7 @@ public class JettyHTTPDestinationTest extends Assert {
                      EndpointReferenceUtils.getAddress(ref),
                      StringUtils.addDefaultPortIfMissing(EndpointReferenceUtils.getAddress(address)));
         assertEquals("unexpected service name local part",
-                     EndpointReferenceUtils.getServiceName(ref).getLocalPart(),
+                     EndpointReferenceUtils.getServiceName(ref, bus).getLocalPart(),
                      "Service");
         assertEquals("unexpected portName",
                      EndpointReferenceUtils.getPortName(ref),
@@ -709,8 +709,9 @@ public class JettyHTTPDestinationTest extends Assert {
                 EasyMock.expect(request.getInputStream()).andReturn(is);
                 EasyMock.expect(request.getContextPath()).andReturn("/bar");
                 EasyMock.expect(request.getPathInfo()).andReturn("/foo");
+                EasyMock.expect(request.getRequestURI()).andReturn("/foo");
                 EasyMock.expect(request.getCharacterEncoding()).andReturn("UTF-8");
-                EasyMock.expect(request.getQueryString()).andReturn(query);            
+                EasyMock.expect(request.getQueryString()).andReturn(query);    
                 EasyMock.expect(request.getHeader("Accept")).andReturn("*/*");  
                 EasyMock.expect(request.getContentType()).andReturn("text/xml charset=utf8");
                 EasyMock.expect(request.getAttribute("org.mortbay.jetty.ajax.Continuation")).andReturn(null);
@@ -739,6 +740,8 @@ public class JettyHTTPDestinationTest extends Assert {
                     EasyMock.expectLastCall();
                     response.addHeader(EasyMock.isA(String.class), EasyMock.isA(String.class));
                     EasyMock.expectLastCall().anyTimes();
+                    response.setContentLength(0);
+                    EasyMock.expectLastCall().anyTimes();
                     response.getOutputStream();
                     EasyMock.expectLastCall().andReturn(os);
                     response.getStatus();
@@ -806,7 +809,7 @@ public class JettyHTTPDestinationTest extends Assert {
         request.getCharacterEncoding();
         EasyMock.expectLastCall().andReturn("UTF-8");
         request.getQueryString();
-        EasyMock.expectLastCall().andReturn("wsdl");       
+        EasyMock.expectLastCall().andReturn("wsdl");    
         response.setContentType("text/xml");
         EasyMock.expectLastCall();        
         response.getOutputStream();
@@ -827,7 +830,10 @@ public class JettyHTTPDestinationTest extends Assert {
         EasyMock.expectLastCall().andReturn(true);   
         wsdlQueryHandler.getResponseContentType("http://localhost/bar/foo?wsdl", "/bar/foo");
         EasyMock.expectLastCall().andReturn("text/xml");
-        wsdlQueryHandler.writeResponse("http://localhost/bar/foo?wsdl", "/bar/foo", endpointInfo, os);
+        wsdlQueryHandler.writeResponse(EasyMock.eq("http://localhost/bar/foo?wsdl"),
+                                       EasyMock.eq("/bar/foo"), 
+                                       EasyMock.eq(endpointInfo),
+                                       (OutputStream)EasyMock.anyObject());
         EasyMock.expectLastCall().once();
         EasyMock.replay(bus);
         EasyMock.replay(queryHandlerRegistry);
