@@ -59,7 +59,7 @@ public class JBITransportFactory extends AbstractTransportFactory implements Con
 
     private Collection<String> activationNamespaces;
 
-    @Resource(name = "bus")
+    @Resource(name = "cxf")
     public void setBus(Bus b) {
         bus = b;
     }
@@ -143,7 +143,7 @@ public class JBITransportFactory extends AbstractTransportFactory implements Con
     public void putDestination(String epName, JBIDestination destination) throws JBIException {
         if (destinationMap.containsKey(epName)) {
             throw new JBIException("JBIDestination for Endpoint "
-                                   + epName + " already be created");
+                                   + epName + " has already been created");
         } else {
             destinationMap.put(epName, destination);
         }
