@@ -19,9 +19,15 @@
 
 package org.apache.cxf.systest.jaxrs;
 
+import java.util.ArrayList;
+import java.util.List;
+
 import javax.ws.rs.GET;
 import javax.ws.rs.Path;
 import javax.ws.rs.ProduceMime;
+import javax.ws.rs.QueryParam;
+import javax.ws.rs.core.Context;
+import javax.ws.rs.core.UriInfo;
 import javax.xml.bind.annotation.XmlRootElement;
 
 
@@ -54,5 +60,30 @@ public class Chapter {
     public Chapter getItself() {
         return this;
     }
+    
+    @GET
+    @ProduceMime("application/xml;charset=ISO-8859-1")
+    public Chapter get() {
+        return this;
+    }
 
+    
+    @GET
+    @Path("/matched-resources")
+    @ProduceMime("text/plain")
+    public String getMatchedResources(@Context UriInfo ui) {
+        List<String> list = new ArrayList<String>();
+        for (Object obj : ui.getAncestorResources()) {
+            list.add(obj.toString());
+        }
+        return list.toString();
+    }
+
+    @GET
+    @Path("/matched%21uris")
+    @ProduceMime("text/plain")
+    public String getMatchedUris(@Context UriInfo ui, 
+                                 @QueryParam("decode") String decode) {
+        return ui.getAncestorResourceURIs(Boolean.parseBoolean(decode)).toString();        
+    }
 }
