@@ -29,10 +29,12 @@ import javax.xml.ws.WebFault;
 
 import org.apache.cxf.common.util.PackageUtils;
 import org.apache.cxf.common.util.StringUtils;
+import org.apache.cxf.tools.common.ToolConstants;
 import org.apache.cxf.tools.common.model.JavaClass;
 import org.apache.cxf.tools.common.model.JavaField;
 import org.apache.cxf.tools.java2wsdl.generator.wsdl11.annotator.WrapperBeanAnnotator;
 import org.apache.cxf.tools.java2wsdl.generator.wsdl11.model.WrapperBeanClass;
+import org.apache.cxf.tools.util.AnnotationUtil;
 import org.apache.cxf.tools.util.URIParserUtil;
 
 public final class FaultBean {
@@ -41,6 +43,18 @@ public final class FaultBean {
                                                                   "getStackTrace",
                                                                   "getClass"};
 
+    public boolean faultBeanExists(final Class<?> exceptionClass) {
+        String fb = getWebFaultBean(exceptionClass);
+        if (!StringUtils.isEmpty(fb)) {
+            try {
+                return AnnotationUtil.loadClass(fb,
+                                                exceptionClass.getClassLoader()) != null;
+            } catch (Exception e) {
+                return false;
+            }
+        }
+        return false; 
+    }
     
     private String getWebFaultBean(final Class<?> exceptionClass) {
         WebFault fault = exceptionClass.getAnnotation(WebFault.class);
@@ -64,10 +78,15 @@ public final class FaultBean {
         }
 
         buildBeanFields(exceptionClass, jClass);
-
-        String pkg = PackageUtils.getPackageName(exceptionClass);
-        jClass.setElementName(new QName(URIParserUtil.getNamespace(pkg),
         
+        String pkg = PackageUtils.getPackageName(exceptionClass);
+        if (pkg.length() > 0) {
+            jClass.setElementName(new QName(URIParserUtil.getNamespace(pkg),
+                                        exceptionClass.getSimpleName()));
+        } else {
+            jClass.setElementName(new QName(URIParserUtil.getNamespace(ToolConstants.DEFAULT_PACKAGE_NAME),
+                                        exceptionClass.getSimpleName()));
+        }
         jClass.annotate(new WrapperBeanAnnotator());
         
         return jClass;
